package com.ipu.config.database;

import com.ipu.config.basic.Resource;
import com.ipu.config.basic.ResourceFinder;
import org.apache.ibatis.session.SqlSession;
import org.apache.ibatis.session.SqlSessionFactory;
import org.apache.ibatis.session.SqlSessionFactoryBuilder;

import java.util.Objects;

public class DatabaseResourceFinder implements ResourceFinder {

    private static SqlSessionFactory sessionFactory;

    static {
        SqlSessionFactoryBuilder builder = new SqlSessionFactoryBuilder();
        sessionFactory = builder.build(DatabaseResourceFinder.class.getResourceAsStream("/configuration.xml"));
    }

    @Override
    public Resource findInitResource(String namespace) {
        Objects.requireNonNull(namespace);

        try (SqlSession sqlSession = sessionFactory.openSession(true)) {
            ResourceMapper resourceMapper = sqlSession.getMapper(ResourceMapper.class);
            return resourceMapper.findInitResource(namespace);
        }
    }

    @Override
    public Resource findDynamicResource(String namespace, String name) {
        Objects.requireNonNull(namespace);
        Objects.requireNonNull(name);

        try (SqlSession sqlSession = sessionFactory.openSession(true)) {
            ResourceMapper resourceMapper = sqlSession.getMapper(ResourceMapper.class);
            return resourceMapper.findDynamicResource(namespace, name);
        }
    }

    @Override
    public Resource findAutoResource(String namespace, String name) {
        Objects.requireNonNull(namespace);
        Objects.requireNonNull(name);

        try (SqlSession sqlSession = sessionFactory.openSession(true)) {
            ResourceMapper resourceMapper = sqlSession.getMapper(ResourceMapper.class);
            return resourceMapper.findAutoResource(namespace, name);
        }
    }
}
